TYPE
  LOGLEVEL : (CRITICAL, WARNING, INFO, DEBUG) := INFO;
END_TYPE

FUNCTION_BLOCK LOGGER
  VAR_INPUT
    TRIG : BOOL;
    MSG : STRING;
    LEVEL : LOGLEVEL := INFO;
  END_VAR
  VAR
    TRIG0 : BOOL;
  END_VAR

  IF TRIG AND NOT TRIG0 THEN
  {{
   LogMessage(GetFbVar(LEVEL),(char*)GetFbVar(MSG, .body),GetFbVar(MSG, .len));
  }}
  END_IF;
  TRIG0:=TRIG;
END_FUNCTION_BLOCK



FUNCTION_BLOCK python_eval
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(0, data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_poll
  VAR_INPUT
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    STATE : DWORD;
    BUFFER : STRING;
    PREBUFFER : STRING;
    TRIGM1 : BOOL;
    TRIGGED : BOOL;
  END_VAR

  {extern void __PythonEvalFB(int, PYTHON_EVAL*);__PythonEvalFB(1,(PYTHON_EVAL*)(void*)data__);}
END_FUNCTION_BLOCK

FUNCTION_BLOCK python_gear
  VAR_INPUT
    N : UINT;
    TRIG : BOOL;
    CODE : STRING;
  END_VAR
  VAR_OUTPUT
    ACK : BOOL;
    RESULT : STRING;
  END_VAR
  VAR
    py_eval : python_eval;
    COUNTER : UINT;
    _TMP_ADD10_OUT : UINT;
    _TMP_EQ13_OUT : BOOL;
    _TMP_SEL15_OUT : UINT;
    _TMP_AND7_OUT : BOOL;
  END_VAR

  _TMP_ADD10_OUT := ADD(COUNTER, 1);
  _TMP_EQ13_OUT := EQ(N, _TMP_ADD10_OUT);
  _TMP_SEL15_OUT := SEL(_TMP_EQ13_OUT, _TMP_ADD10_OUT, 0);
  COUNTER := _TMP_SEL15_OUT;
  _TMP_AND7_OUT := AND(_TMP_EQ13_OUT, TRIG);
  py_eval(TRIG := _TMP_AND7_OUT, CODE := CODE);
  ACK := py_eval.ACK;
  RESULT := py_eval.RESULT;
END_FUNCTION_BLOCK


TYPE
  sCnv : STRUCT
    Lrun : BOOL;
    Rrun : BOOL;
    LsLeft : BOOL;
    LsRight : BOOL;
  END_STRUCT;
  sManu : STRUCT
    RqCnvLeft : BOOL;
    RqCnvRight : BOOL;
  END_STRUCT;
  sIO : STRUCT
    LS1 : BOOL;
    LS2 : BOOL;
    LS3 : BOOL;
    LS4 : BOOL;
    LS5 : BOOL;
    PB1 : BOOL;
    PB2 : BOOL;
    PB3 : BOOL;
    PB4 : BOOL;
    PB5 : BOOL;
    SS1 : BOOL;
    SS0 : BOOL;
    DSW1 : BOOL;
    DSW2 : BOOL;
    DSW4 : BOOL;
    DSW8 : BOOL;
    RY1 : BOOL;
    RY2 : BOOL;
    PL1 : BOOL;
    PL2 : BOOL;
    PL3 : BOOL;
    PL4 : BOOL;
    DP1 : BOOL;
    DP2 : BOOL;
    DP4 : BOOL;
    DP8 : BOOL;
    DP10 : BOOL;
    DP20 : BOOL;
    DP40 : BOOL;
    DP80 : BOOL;
    SPD0 : BOOL;
    SPD1 : BOOL;
  END_STRUCT;
END_TYPE

PROGRAM InterfaceOUT
  VAR_EXTERNAL
    G_sIO : sIO;
  END_VAR
  VAR
    out_0 : BOOL;
    out_1 : BOOL;
    out_2 : BOOL;
    out_3 : BOOL;
    out_4 : BOOL;
    out_5 : BOOL;
    out_6 : BOOL;
    out_7 : BOOL;
    out_8 : BOOL;
    out_9 : BOOL;
    out_10 : BOOL;
    out_11 : BOOL;
    out_12 : BOOL;
    out_13 : BOOL;
    out_14 : BOOL;
  END_VAR

  out_0 := G_sIO.RY1;
  out_1 := G_sIO.RY2;
  out_2 := G_sIO.PL1;
  out_3 := G_sIO.PL2;
  out_4 := G_sIO.PL3;
  out_5 := G_sIO.PL4;
  out_6 := G_sIO.DP1;
  out_7 := G_sIO.DP2;
  out_8 := G_sIO.DP4;
  out_9 := G_sIO.DP8;
  out_10 := G_sIO.SPD0;
  out_11 := G_sIO.DP10;
  out_12 := G_sIO.DP20;
  out_13 := G_sIO.DP40;
  out_14 := G_sIO.DP80;
END_PROGRAM

PROGRAM ConverObj
  VAR_EXTERNAL
    G_sCnv : sCnv;
    G_sManu : sManu;
    G_sIO : sIO;
  END_VAR

  G_sCnv.Lrun := G_sManu.RqCnvLeft;
  G_sCnv.Rrun := G_sManu.RqCnvRight;
  G_sCnv.LsLeft := G_sIO.LS2;
  G_sCnv.LsRight := G_sIO.LS1;
  G_sIO.RY1 := G_sCnv.Lrun AND NOT(G_sCnv.LsLeft);
  G_sIO.RY2 := G_sCnv.Rrun AND NOT(G_sCnv.LsRight);
END_PROGRAM

PROGRAM LampObj
  VAR_EXTERNAL
    G_sIO : sIO;
    G_sCnv : sCnv;
  END_VAR
  VAR
    Pls1sec : BOOL;
    TON0 : TON;
    FF1sec : BOOL;
    FFres : BOOL;
  END_VAR

  TON0(IN := NOT(Pls1sec), PT := TIME#1000ms);
  Pls1sec := TON0.Q;
  FFres := FF1sec AND Pls1sec;
  FF1sec := NOT(FFres) AND (FF1sec OR Pls1sec);
  G_sIO.PL1 := G_sCnv.LsLeft AND G_sCnv.Lrun AND FF1sec OR NOT(G_sCnv.LsLeft) AND G_sCnv.Lrun;
  G_sIO.PL2 := G_sCnv.LsRight AND G_sCnv.Rrun AND FF1sec OR NOT(G_sCnv.LsRight) AND G_sCnv.Rrun;
END_PROGRAM

PROGRAM InterfaceIN
  VAR_EXTERNAL
    G_sIO : sIO;
  END_VAR
  VAR
    in_0 : BOOL;
    in_1 : BOOL;
    in_2 : BOOL;
    in_3 : BOOL;
    in_4 : BOOL;
    in_5 : BOOL;
    in_6 : BOOL;
    in_7 : BOOL;
    in_8 : BOOL;
    in_9 : BOOL;
    in_10 : BOOL;
    in_11 : BOOL;
    in_12 : BOOL;
    in_13 : BOOL;
    in_14 : BOOL;
    in_15 : BOOL;
  END_VAR

  G_sIO.LS1 := in_0;
  G_sIO.LS2 := in_1;
  G_sIO.LS3 := in_2;
  G_sIO.LS4 := in_3;
  G_sIO.LS5 := in_4;
  G_sIO.PB1 := in_5;
  G_sIO.PB2 := in_6;
  G_sIO.PB3 := in_7;
  G_sIO.PB4 := in_8;
  G_sIO.PB5 := in_9;
  G_sIO.SS1 := in_10;
  G_sIO.SS0 := in_11;
  G_sIO.DSW1 := in_12;
  G_sIO.DSW2 := in_13;
  G_sIO.DSW4 := in_14;
  G_sIO.DSW8 := in_15;
END_PROGRAM

PROGRAM ManuCont
  VAR_EXTERNAL
    G_sManu : sManu;
    G_sIO : sIO;
  END_VAR

  G_sManu.RqCnvLeft := G_sIO.PB1;
  G_sManu.RqCnvRight := G_sIO.PB2;
END_PROGRAM


CONFIGURATION Config0
  VAR_GLOBAL
    G_sIO : sIO;
    G_sCnv : sCnv;
    G_sManu : sManu;
  END_VAR

  RESOURCE Res0 ON PLC
    TASK task0(INTERVAL := T#20ms,PRIORITY := 0);
    PROGRAM instance0 WITH task0 : InterfaceIN;
    PROGRAM instance1 WITH task0 : ManuCont;
    PROGRAM instance2 WITH task0 : ConverObj;
    PROGRAM instance3 WITH task0 : LampObj;
    PROGRAM instance4 WITH task0 : InterfaceOUT;
  END_RESOURCE
END_CONFIGURATION
